import bluebelt.helpers.defaults as defaults

# main plot | pyplot.plot
plot = {
    'marker': None,
    'zorder': 20,
    'linestyle': '-',
    'linewidth': 1,
}

# plot title | pyplot.set_title
title = {
    'loc': 'left',
    'size': defaults.medium
}
