import bluebelt.helpers.defaults as defaults

# boxplot | pyplot.boxplot
boxplot = {
    'boxes': {
        'hatch': '////',
        },
}

# plot title | pyplot.set_title
title = {
    'loc': 'left',
}